<?php
namespace Zeedhi\Notification;

/**
 * class API
 *
 * Implements all request methods to Zeedhi Notification API required by the application server
 *
 * @package Zeedhi\Notification
 */
class API {

    /** @var string */
    const ZEEDHI_NOTIFICATION_URL = 'http://notification.zeedhi.com';
    /** @var string Zeedhi API Key */
    private $zeedhiApiKey;
    /** @var string API Key provided by Notification Manager */
    private $applicationId;
    /** @var cURLRequest Implementation of request strategy */
    private $cURLRequest;

    /**
     * Construct
     *
     * @param string      $zeedhiApiKey  Zeedhi API Key
     * @param string      $applicationId API Key provided by Notification Manager
     * @param cURLRequest $cURLRequest   Implementation of request strategy
     */
    public function __construct($zeedhiApiKey, $applicationId, $cURLRequest) {
        $this->zeedhiApiKey  = $zeedhiApiKey;
        $this->applicationId = $applicationId;
        $this->cURLRequest = $cURLRequest;
    }

    /**
     * factoryMessagesFromRows
     *
     * Convert array to Message objects
     *
     * @param array $messageRows
     * @return Message[]
     */
    protected function factoryMessagesFromRows(array $messageRows) {
        $messages = array();
        foreach ($messageRows as $messageRow) {
            $messages[] = new Message(
                $messageRow['_id'],
                $messageRow['topicId'],
                $messageRow['notification'],
                $messageRow['nrReceived'],
                $messageRow['nrRead']
            );
        }

        return $messages;
    }

    /**
     * factoryTopicsFromRows
     *
     * Convert array to Topic objects
     *
     * @param array $topicRows
     *
     * @return Topic[]
     */
    protected function factoryTopicsFromRows(array $topicRows) {
        $topics = array();
        foreach ($topicRows as $topicRow) {
            $topics[] = new Topic($topicRow['_id'], $topicRow['name'], $topicRow['nrSubscribers']);
        }

        return $topics;
    }

    /**
     * request
     *
     * Do request using implementation strategy
     *
     * @param string $requestPath
     * @param array  $postFields
     *
     * @return array
     *
     * @throws Exception
     */
    protected function request($requestPath, $postFields = array()) {
        $postFields['applicationId'] = $this->applicationId;
        $this->cURLRequest->setHeaders(array(
            'zhApiKey' => $this->zeedhiApiKey,
            'Content-type' => 'application/x-www-form-urlencoded'
        ));
        $response = $this->cURLRequest->request($requestPath, $postFields);
        $output = json_decode($response, true);
        if (isset($output['error']) && $output['error']) {
            throw Exception::requestError($output['error']);
        }

        return $output;
    }

    /**
     * sendMessage
     *
     * Send a message to topic.
     *
     * @param string $topicName
     * @param array  $notification A array with keys(icon, title, message, data).
     *
     * @return Message
     */
    public function sendMessage($topicName, array $notification) {
        $output = $this->request('/message/send/', array(
            'topicName'     => $topicName,
            'notification'  => json_encode($notification)
        ));

        return current($this->factoryMessagesFromRows($output['dataset']['message']));
    }

    /**
     * getAllTopics
     *
     * Get all topics from application
     *
     * @return Topic[]
     */
    public function getAllTopics() {
        $output = $this->request('/topic/');
        return $this->factoryTopicsFromRows($output['dataset']['topic']);
    }

    /**
     * getAllMessages
     *
     * Get all messages from topic
     *
     * @param string $topicName
     *
     * @return Message[]
     */
    public function getAllMessages($topicName) {
        $output = $this->request('/message/', array('topicName' => $topicName));
        return $this->factoryMessagesFromRows($output['dataset']['message']);
    }

    /**
     * createTopic
     *
     * @param string $topicName
     *
     * @return Topic
     *
     * @throws Exception
     */
    public function createTopic($topicName) {
        $output = $this->request('/topic/create/', array('topicName' => $topicName));
        return current($this->factoryTopicsFromRows($output['dataset']['topic']));
    }

    /**
     * factoryAPI
     *
     * Creates API object with default parameters
     *
     * @param string $zeedhiApiKey  API Key
     * @param string $applicationId Application ID provided by Notification Manager
     *
     * @return API Instance of API Class
     */
    public static function factoryAPI($zeedhiApiKey, $applicationId) {
        return new static($zeedhiApiKey, $applicationId, new cURLRequest(self::ZEEDHI_NOTIFICATION_URL));
    }
}